﻿XIncludeFile "..\functions.pbi"
XIncludeFile "..\profiles.pbi"
XIncludeFile "JsCompressor.pbf"

#APPNAME = "JsCompressor"
#Version = "2014-12-15"

#C = "c"
#F = "f"

EnableExplicit

Structure sCompilerInfo
	TargetFolder.s
	IndexPage.s
	ServerAddress.s
	CgiFilename.s
	CgiAddress.s
	ApplicationName.s
	Encoding.s
	JQueryJs.s
	JQueryUiJs.s
	JQueryUiCss.s
	JQueryMobileJs.s
	JQueryMobileCss.s
	AdditionalScriptsAndStylesheets.s
	DoNotCopy.i
	ProfileName.s
	IsMobile.i
EndStructure

Global CompilerInfo.sCompilerInfo

Global AppDataPath.s = GetAppDataPath("tuebbentools", "PB2Web")

Global IniPath.s = AppDataPath + "PB2Web.ini"

Global MinifyJsCmd.s

Procedure.s Get_npm_Path()
	
	Protected pathes.s, path.s, npmPath.s
	Protected Counter
	
	; Find npm-Path
	
	pathes.s = GetEnvironmentVariable("path")
	For Counter = 1 To CountString(pathes, ";") + 1
		path.s = StringField(pathes, Counter, ";") + #PathSeparator
		If FindString(path, #PathSeparator + "npm" + #PathSeparator, 1, #PB_String_NoCase)
			npmPath.s = path
			Break
		EndIf
	Next
	
	ProcedureReturn npmPath
	
EndProcedure

Procedure AddMessageToLog(Log.s)
	
	AddGadgetItem(#frmMain_Log, -1, Log)

EndProcedure

Procedure Compress(Dummy)
	
	PostEvent(#EventBeginProcessing)

	Structure sCCC
		Constant.s
		Value.s
	EndStructure
	
	NewList Constant.sCCC()
	
	Protected File_PB2WebJs.s
	Protected File_ApplicationJs.s
	
	Protected PB2WebJs.s
	Protected ApplicationJs.s
	
	File_PB2WebJs      = CompilerInfo\TargetFolder + #PathSeparator + "libraries" + #PathSeparator + "pb2web" + #PathSeparator + "pb2web.js"
	File_ApplicationJs = CompilerInfo\TargetFolder + #PathSeparator + CompilerInfo\ApplicationName + ".js"
	
	PB2WebJs      = LoadTextFile(File_PB2WebJs)
	ApplicationJs = LoadTextFile(File_ApplicationJs)
	
	If FindString(PB2WebJs, "/*cmprsd*/")
		AddMessageToLog("Files are already compressed...")
		AddMessageToLog("Ready! :-)")
		PostEvent(#EventProcessingFinished)
		ProcedureReturn
	EndIf
	
	Protected Line.s
	Protected ConstantName.s
	Protected C2.s
	
	Protected RegEx
	Protected Constants.s
	
	; Replace Constants
	
	AddMessageToLog("Replacing Constants...")
	
	RegEx = CreateRegularExpression(#PB_Any, "var cConstants = \(function \(\) {([\s\S]*?)}\)\(\);")
	
	If RegEx
		
		If ExamineRegularExpression(RegEx, PB2WebJs)
			While NextRegularExpressionMatch(RegEx)
				Constants = RegularExpressionMatchString(RegEx)
				PB2WebJs  = ReplaceRegularExpression(RegEx, PB2WebJs, "")
			Wend
		EndIf
		
		FreeRegularExpression(RegEx)
		
	EndIf
	
	If Constants <> ""
		
		RegEx = CreateRegularExpression(#PB_Any, "cConstants\.([A-Za-z0-9_]*)(\s)=(\s)(.*)")
		
		If RegEx
			
			If ExamineRegularExpression(RegEx, Constants)
				While NextRegularExpressionMatch(RegEx)
					
					Line = RegularExpressionMatchString(RegEx)
					
					ConstantName = Trim(StringField(Line, 1, "="))
					
					If ConstantName = "cConstants.PB_Compiler_EnumerationValue"
						Continue
					EndIf
					
					C2 = Trim(StringField(Line, 2, "="))
					
					C2 = RemoveString(C2, ";")
					C2 = RemoveString(C2, #CR$)
					C2 = RemoveString(C2, #LF$)
					
					AddElement(Constant())
					
					Constant()\Constant = ConstantName
					Constant()\Value    = C2
					
				Wend
			EndIf
			
			FreeRegularExpression(RegEx)
			
		EndIf
		
	EndIf
	
	SortStructuredList(Constant(), #PB_Sort_Descending, OffsetOf(sCCC\Constant), TypeOf(sCCC\Constant))
	
	ForEach Constant()
		
		PB2WebJs = ReplaceString(PB2WebJs, Constant()\Constant, Constant()\Value)
		
	Next
	
	; ----
	
	; Remove Debugger-Output
	
	AddMessageToLog("Removing Debugger-Output...")
	
	RegEx = CreateRegularExpression(#PB_Any, "cDebugger\.Debug\([\s\S]*?\);")
	
	If RegEx
		
		If ExamineRegularExpression(RegEx, PB2WebJs)
			While NextRegularExpressionMatch(RegEx)
				PB2WebJs  = ReplaceRegularExpression(RegEx, PB2WebJs, "")
			Wend
		EndIf
		
		FreeRegularExpression(RegEx)
		
	EndIf
	
	; ----
	
	; Remove 'NotImplemented'-Messages
	
	AddMessageToLog("Removing 'NotImplemented'-Messages...")
	
	RegEx = CreateRegularExpression(#PB_Any, "cCommon\.NotImplemented\([\s\S]*?\);")
	
	If RegEx
		
		If ExamineRegularExpression(RegEx, PB2WebJs)
			While NextRegularExpressionMatch(RegEx)
				PB2WebJs  = ReplaceRegularExpression(RegEx, PB2WebJs, "")
			Wend
		EndIf
		
		FreeRegularExpression(RegEx)
		
	EndIf
	
	; ----
	
	; Replace Classes & Functions
	
	AddMessageToLog("Replacing Classes & Functions...")
	
	Structure sClassMap
		ClassName.s
		Map Function.s()
	EndStructure
	
	NewMap ClassMap.sClassMap()
	
	RegEx = CreateRegularExpression(#PB_Any, "\s[A-Za-z0-9_]*\.[A-Za-z0-9_]*\s=\sfunction\s\(")
	
	Protected ClassName.s
	Protected FunctionName.s
	
	If RegEx
		
		If ExamineRegularExpression(RegEx, PB2WebJs)
			While NextRegularExpressionMatch(RegEx)
				
				Line = RegularExpressionMatchString(RegEx)     ; c2DDrawing.Alpha = function (
				Line = Trim(StringField(Line, 1, "="))         ; c2DDrawing.Alpha
				ClassName = Trim(StringField(Line, 1, "."))    ; c2DDrawing
				FunctionName = Trim(StringField(Line, 2, ".")) ; Alpha
				
				ClassMap(ClassName)\ClassName = ClassName
				ClassMap(ClassName)\Function(FunctionName) = FunctionName
				
			Wend
		EndIf
		
		FreeRegularExpression(RegEx)
		
	EndIf
	
	; --- 
	
	; Transfer Informations from Map to LinkedList (in order to sort the list)
	
	Structure sClassLinkedList
		ClassName.s
		List Function.s()
	EndStructure
	
	NewList ClassLinkedList.sClassLinkedList()
	
	ForEach ClassMap()
		AddElement(ClassLinkedList())
		ClassLinkedList()\ClassName = ClassMap()\ClassName
		ForEach ClassMap()\Function()
			AddElement(ClassLinkedList()\Function())
			ClassLinkedList()\Function() = ClassMap()\Function()
		Next
	Next
	
	FreeMap(ClassMap()) ; the map is no longer needed
	
	SortStructuredList(ClassLinkedList(), #PB_Sort_Descending, OffsetOf(sClassLinkedList\ClassName), TypeOf(sClassLinkedList\ClassName))
	
	ForEach ClassLinkedList()
		SortList(ClassLinkedList()\Function(), #PB_Sort_Descending)
	Next
	
	; ---
	
	; Replace Class- & Functionnames
	
	ForEach ClassLinkedList()
		
		PB2WebJs = ReplaceString(PB2WebJs, "var " + ClassLinkedList()\ClassName + " = (function () {", "var " + #C + Str(ListIndex(ClassLinkedList())) + " = (function () {")
		PB2WebJs = ReplaceString(PB2WebJs, "function " + ClassLinkedList()\ClassName + "(", "function " + #C + Str(ListIndex(ClassLinkedList())) + "(")
		PB2WebJs = ReplaceString(PB2WebJs, "return " + ClassLinkedList()\ClassName + ";", "return " + #C + Str(ListIndex(ClassLinkedList())) + ";")
		
		ForEach ClassLinkedList()\Function()
			PB2WebJs = ReplaceString(PB2WebJs, ClassLinkedList()\ClassName + "." + ClassLinkedList()\Function(), #C + Str(ListIndex(ClassLinkedList())) + "." + #F + Str(ListIndex(ClassLinkedList()\Function())))
			PB2WebJs = ReplaceString(PB2WebJs, "this." + ClassLinkedList()\Function(), #C + Str(ListIndex(ClassLinkedList())) + "." + #F + Str(ListIndex(ClassLinkedList()\Function())))
		Next
		
		PB2WebJs = ReplaceString(PB2WebJs, ClassLinkedList()\ClassName + ".", #C + Str(ListIndex(ClassLinkedList())) + ".")
		
	Next
	
	; ----
	
	SaveTextFile(PB2WebJs, File_PB2WebJs)
	
	; ------------
	
	
	ForEach Constant()
		ApplicationJs = ReplaceString(ApplicationJs, Constant()\Constant, Constant()\Value)
	Next
	
	ForEach ClassLinkedList()
		ForEach ClassLinkedList()\Function()
			ApplicationJs = ReplaceString(ApplicationJs, ClassLinkedList()\ClassName + "." + ClassLinkedList()\Function(), #C + Str(ListIndex(ClassLinkedList())) + "." + #F + Str(ListIndex(ClassLinkedList()\Function())))
		Next
		ApplicationJs = ReplaceString(ApplicationJs, ClassLinkedList()\ClassName + ".", #C + Str(ListIndex(ClassLinkedList())) + ".")
	Next
	
	SaveTextFile(ApplicationJs, File_ApplicationJs)
	
	Protected Minify = #True
	
	If Minify
		
		; ----
		
		Protected RunningProgram
		Protected ProgramError.s
		Protected ProgramString.s
		; 
		
		AddMessageToLog("Minifying PB2Web.js...")
		
		RunningProgram = RunProgram(MinifyJsCmd, "-e best -m -o " + Chr(34) + File_PB2WebJs + ".bak" + Chr(34) + " -i " + Chr(34) + File_PB2WebJs + Chr(34), "", #PB_Program_Open | #PB_Program_Read | #PB_Program_Error | #PB_Program_Hide)
		
		If RunningProgram
			
			While ProgramRunning(RunningProgram)
				
				ProgramError = ReadProgramError(RunningProgram)
				If ProgramError
					Debug ProgramError
					AddMessageToLog(ProgramError)
				EndIf
				
				ProgramString = ReadProgramString(RunningProgram)
				If ProgramString
					AddMessageToLog(ProgramString)
				EndIf
				
			Wend
			
			CloseProgram(RunningProgram)
			
			; ----
			
			DeleteFile(File_PB2WebJs)
			CopyFile(File_PB2WebJs + ".bak", File_PB2WebJs)
			DeleteFile(File_PB2WebJs + ".bak")
			
			; ---
			
			OpenFile(0, File_PB2WebJs, #PB_File_Append)
			WriteString(0, "/*cmprsd*/")
			CloseFile(0)
		
		Else
			
			AddMessageToLog("Couldn't run minifyjs.cmd")
			
		EndIf
		
		; ----
		
		AddMessageToLog("Minifying " + CompilerInfo\ApplicationName + ".js...")
		
		RunningProgram = RunProgram(MinifyJsCmd, "-e best -m -o " + Chr(34) + File_ApplicationJs + ".bak" + Chr(34) + " -i " + Chr(34) + File_ApplicationJs + Chr(34), "", #PB_Program_Open | #PB_Program_Read | #PB_Program_Error | #PB_Program_Hide)
		
		If RunningProgram
			
			While ProgramRunning(RunningProgram)
				
				ProgramError = ReadProgramError(RunningProgram)
				If ProgramError
					AddMessageToLog(ProgramError)
				EndIf
				
				ProgramString = ReadProgramString(RunningProgram)
				If ProgramString
					AddMessageToLog(ProgramString)
				EndIf
				
			Wend
			
			CloseProgram(RunningProgram)
			
			DeleteFile(File_ApplicationJs)
			CopyFile(File_ApplicationJs + ".bak", File_ApplicationJs)
			DeleteFile(File_ApplicationJs + ".bak")
			
		Else
			
			AddMessageToLog("Couldn't run minifyjs.cmd")
			
		EndIf
		
	EndIf

	AddMessageToLog("Ready! :-)")
	PostEvent(#EventProcessingFinished)
	
EndProcedure

Procedure frmMain_cboProfiles_Event()
	
	ForEach Profile()
		
		If Profile()\Name = GetGadgetText(#frmMain_cboProfiles)

			ClearGadgetItems(#frmMain_Log)
			
			AddGadgetItem(#frmMain_Log, -1, "Serverpath: " + Profile()\ServerPath)
			
			If Profile()\ApplicationName = ""
				AddGadgetItem(#frmMain_Log, -1, "ApplicationName: application")
			Else
				AddGadgetItem(#frmMain_Log, -1, "ApplicationName: " + Profile()\ApplicationName)
			EndIf			
			
			Break	
			
		EndIf
		
	Next  	
	
EndProcedure

Procedure frmMain_cmdOK_Event()
	
	ForEach Profile()
		
		If Profile()\Name = GetGadgetText(#frmMain_cboProfiles)

			CompilerInfo\TargetFolder = Profile()\ServerPath
			CompilerInfo\ApplicationName = Profile()\ApplicationName
			
			If CompilerInfo\ApplicationName = ""
				CompilerInfo\ApplicationName = "application"
			EndIf			
			
			ClearGadgetItems(#frmMain_Log)
			
			CreateThread(@Compress(), 42)
			
			Break
			
		EndIf
		
	Next
	
EndProcedure

OpenfrmMain()

LoadProfiles(IniPath)

Define LastProfile.s

OpenPreferences(IniPath)
LastProfile = ReadPreferenceString("JsCompressor_LastProfile", "")
ClosePreferences()

ForEach Profile()
	AddGadgetItem(#frmMain_cboProfiles, -1, Profile()\Name)
	If LastProfile = Profile()\Name
		SetGadgetState(#frmMain_cboProfiles, CountGadgetItems(#frmMain_cboProfiles) - 1)
	EndIf
Next

If GetGadgetState(#frmMain_cboProfiles) = -1
	SetGadgetState(#frmMain_cboProfiles, 0)
EndIf

BindGadgetEvent(#frmMain_cboProfiles, @frmMain_cboProfiles_Event())
BindGadgetEvent(#frmMain_cmdOK, @frmMain_cmdOK_Event())

frmMain_cboProfiles_Event()

AddMessageToLog("---")

MinifyJsCmd = Get_npm_Path() + "minifyjs.cmd"

If FileSize(MinifyJsCmd) = -1
	AddMessageToLog(MinifyJsCmd + ": Not found :-(")
Else
	AddMessageToLog(MinifyJsCmd + ": found :-)")
EndIf

Define WWE
Define ThreadIsRunning

Repeat
	
	WWE = WaitWindowEvent()
	
	Select WWE
	
		Case #EventBeginProcessing					
			
			ThreadIsRunning = #True
			
			DisableGadget(#frmMain_cboProfiles, #True)
			DisableGadget(#frmMain_cmdOK, #True)
			DisableGadget(#frmMain_Log, #True)
			
		Case #EventProcessingFinished
			
			ThreadIsRunning = #False
			
			DisableGadget(#frmMain_cboProfiles, #False)
			DisableGadget(#frmMain_cmdOK, #False)
			DisableGadget(#frmMain_Log, #False)
			
		Case #PB_Event_CloseWindow
			
			If Not ThreadIsRunning
				
				Break
				
			EndIf
			
	EndSelect	
	
ForEver

If OpenPreferences(IniPath)
	WritePreferenceString("JsCompressor_LastProfile", GetGadgetText(#frmMain_cboProfiles))
	ClosePreferences()
EndIf



; IDE Options = PureBasic 5.31 (Windows - x86)
; CursorPosition = 404
; Folding = w6
; EnableUnicode
; EnableThread
; EnableXP
; Executable = ..\JsCompressor.exe
; IncludeVersionInfo
; VersionField0 = 2014.12.15
; VersionField1 = 2014.12.15
; VersionField2 = tuebbentools
; VersionField3 = JsCompressor
; VersionField4 = 2014.12.15
; VersionField5 = 2014.12.15
; VersionField6 = a tool to compress JavaScript
; VersionField7 = JsCompressor
; VersionField8 = JsCompressor
; VersionField13 = tuebben@gmail.com
; VersionField14 = http://pb2web.de/