EnableExplicit

! var arrDTS = [];

Procedure MonacoEditorGadget(Gadget, x, y, Width, Height, Flags = 0, Callback = 0)
	
	!	var Gadget = cCommon.SetGadgetAndStuff(Gadget);
	
	!	var selector = $("<div />");
	
	!	selector.attr("id", "g_" + Gadget);
	
	!	cCommon.SetGadgetPosition(selector, x, y, Width, Height);
	
	!	selector.data("GadgetType", "MonacoEditorGadget");
	
	!	cCommon.CurrentContainer.append(selector);
	
	
	!	require.config({ paths: { 'vs': 'userlibraries/monaco-editor/min/vs' }});
	
	!	require(['vs/editor/editor.main'], function() {
	
	!		editor = monaco.editor.create(document.getElementById('g_' + Gadget), {
	!			fontFamily: 'consolas',
	!			language: 'typescript',
	!			folding: true
	!		});
	
	!		selector.data("editor", editor);
	
	!		if (typeof Callback === "function") Callback();
	
	!	});
	
	
	!	return Gadget;
	
EndProcedure

Procedure MonacoEditorGadget_SetGadgetText(Gadget, Text.s)
	
	!	var selector = $("#g_" + Gadget);	
	!	if (selector.length === 0) return cCommon.GadgetIsNotInitialized("SetGadgetText");
	!	var editor = selector.data("editor");
	!	editor.setValue(Text);
	
EndProcedure
RegisterUserGadgetProcedure("MonacoEditorGadget", "SetGadgetText", @MonacoEditorGadget_SetGadgetText())

Procedure.s MonacoEditorGadget_GetGadgetText(Gadget)
	
	!	var selector = $("#g_" + Gadget);	
	!	if (selector.length === 0) return cCommon.GadgetIsNotInitialized("GetGadgetText");
	!	var editor = selector.data("editor");
	!	return editor.getValue();
	
EndProcedure
RegisterUserGadgetProcedure("MonacoEditorGadget", "GetGadgetText", @MonacoEditorGadget_GetGadgetText())

Procedure MonacoEditorGadget_ResizeGadget(Gadget, x, y, Width, Height)
	
	!	var selector = $("#g_" + Gadget);	
	!	if (selector.length === 0) return cCommon.GadgetIsNotInitialized("ResizeGadget");
	
	!	if (x !== cConstants.PB_Ignore) selector.css("left", x);
	!	if (y !== cConstants.PB_Ignore) selector.css("top", y);
	!	if (Width !== cConstants.PB_Ignore) selector.css("width", Width);	
	!	if (Height !== cConstants.PB_Ignore) selector.css("height", Height);	
	
	!	var editor = selector.data("editor");
	!	editor.layout();
	
EndProcedure
RegisterUserGadgetProcedure("MonacoEditorGadget", "ResizeGadget", @MonacoEditorGadget_ResizeGadget())

Procedure FormatCode(Gadget)
	!	var selector = $("#g_" + Gadget);	
	!	if (selector.length === 0) return cCommon.GadgetIsNotInitialized("FormatCode");
	!	var editor = selector.data("editor");
	!	editor.getAction('editor.action.format').run();
EndProcedure

Procedure MonacoEditorRefresh(Gadget)
	!	var selector = $("#g_" + Gadget);	
	!	if (selector.length === 0) return cCommon.GadgetIsNotInitialized("FormatCode");
	!	var editor = selector.data("editor");
	! var model = editor.getModel();
	!	editor.dispose();
	!	editor = monaco.editor.create(document.getElementById('g_' + Gadget), {
	!		fontFamily: 'consolas',
	!		language: 'typescript',
	!		folding: true
	!	});
	!	selector.data("editor", editor);	
	! editor.setModel(model);
	! var bla = 42;
EndProcedure

Procedure SetMonacoTheme(Gadget, Theme.s)
	!	var selector = $("#g_" + Gadget);	
	!	if (selector.length === 0) return cCommon.GadgetIsNotInitialized("SetMonacoTheme");
	!	var editor = selector.data("editor");
	!	editor.updateOptions({ 'theme': Theme });
EndProcedure

Procedure RemoveTypescriptDeclaration(filename.s)
	!	if (typeof arrDTS[filename] != 'undefined') { 
	! 	cDebugger.Debug("arrDTS[" + filename + "].dispose()");
	! 	arrDTS[filename].dispose();
	! }
EndProcedure

Procedure AddTypescriptDeclaration(dts.s, filename.s)
	RemoveTypescriptDeclaration(filename)
	!	arrDTS[filename] = monaco.languages.typescript.typescriptDefaults.addExtraLib(dts, filename);
EndProcedure


; IDE Options = PureBasic 5.50 (Windows - x86)
; CursorPosition = 94
; FirstLine = 21
; Folding = x-
; EnableXP