<?php

/**
 * cString short summary.
 *
 * cString description.
 *
 * @version 1.0
 * @author Peter
 */

require_once("cConstants.php");

class cString {

    // Result = Asc(String$)
    static function Asc($String) {
        return ord($String);
    }
    
    // Result$ = Bin(Value.q [, Type])
    static function Bin($Value, $Type = 0) {
        // ToDo: $Type
        return decbin($Value);
    }
    
    // Text$ = Chr(CharacterValue)
    static function Chr($CharacterValue) {
        return chr($CharacterValue);
    }
    
    // Result = CountString(String$, StringToCount$)
    static function CountString($String, $StringToCount) {
        return substr_count($String, $StringToCount);
    }
    
    // Position = FindString(String$, StringToFind$ [, StartPosition [, Mode]])
    static function FindString($String, $StringToFind, $StartPosition = 1, $Mode = cConstants::PB_String_CaseSensitive) {
        
        // #PB_String_CaseSensitive: case sensitive search (a=a) (default).
        // #PB_String_NoCase       : case insensitive search (A=a).
        
		if ($StartPosition < 1) $StartPosition = 1;

		$StartPosition -= 1;

		switch ($Mode) {

			case cConstants::PB_String_CaseSensitive:
				return strpos($String, $StringToFind, $StartPosition) + 1;

            case cConstants::PB_String_NoCase:
				return strpos(strtolower($String), strtolower($StringToFind), $StartPosition) + 1;

			default:
				return -1;        
            
        }        
        
    }
    
    // Result$ = Hex(Value.q [, Type])
    static function Hex($Value, $Type = 0) {
        // ToDo: $Type
        return dechex($Value);
    }
    
    // Result$ = InsertString(String$, StringToInsert$, Position)
    static function InsertString($String, $StringToInsert, $Position) {
        $Position -= 1;
        return substr_replace($String, $StringToInsert, $Position, 0);
    }
    
    // Result$ = LCase(String$)
    static function LCase($String) {
        return strtolower($String);
    }

    // Result$ = LSet(String$, Length [, Character$])
    static function LSet($String, $Length, $Character = " ") {
        return str_pad($String, $Length, $Character, STR_PAD_LEFT);
    }

    // Result$ = LTrim(String$ [, Character$])
    static function LTrim($String, $Character = "") {
        return ltrim($String, $Character);
    }

    // Result$ = Left(String$, Length)
    static function Left($String, $Length) {
        return substr($String, 0, $Length);
    }

    // Length = Len(String$)
    static function Len($String) {
        return strlen($String);
    }

    // Result$ = Mid(String$, StartPosition [, Length])
    static function Mid($String, $StartPosition, $Length = -1) {
        $StartPosition -= 1;
        if ($Length==-1) $Length = strlen($String);
        return substr($String, $StartPosition, $Length);
    }

    // Result$ = RSet(String$, Length [, Character$])
    static function RSet($String, $Length, $Character = " ") {
        return str_pad($String, $Length, $Character);
    }

    // Result$ = RTrim(String$ [, Character$])
    public function RTrim($String, $Character = "") {
        return rtrim($String, $Character);
    }

    // String$ = RemoveString(String$, StringToRemove$ [, Mode [, StartPosition [, NbOccurrences]]])
    static function RemoveString($String, $StringToRemove, $Mode = cConstants::PB_String_CaseSensitive, $StartPosition = 0, $NbOccurrences = 0) {
		
        // #PB_String_CaseSensitive: case sensitive search (a=a). Default value if the parameter is omitted.
        // #PB_String_NoCase       : case insensitive remove (A=a).

        return cString::ReplaceString($String, $StringToRemove, "", $Mode, $StartPosition, $NbOccurrences);
        
    }

    // String$ = ReplaceString(String$, StringToFind$, StringToReplace$ [, Mode [, StartPosition [, NbOccurrences]]])
    static function ReplaceString($String, $StringToFind, $StringToReplace, $Mode = cConstants::PB_String_CaseSensitive, $StartPosition = 1, $NbOccurrences = -1) {
        
        /*
		#PB_String_CaseSensitive: case sensitive search (a=a). Default value if the parameter is omitted.
		#PB_String_NoCase       : case insensitive remove (A=a).
		*/

		if (empty($String)) return "";
		if (empty($StringToFind)) return $String;

		if ($StringToFind == $StringToReplace) return $String;

		if ($StartPosition > strlen($String)) return $String;
		if ($StartPosition <= 0) $StartPosition = 1;

		if ($NbOccurrences == -1) $NbOccurrences = strlen($String);

		$p1 = cString::Left($String, $StartPosition - 1);
		$p2 = cString::Mid($String, $StartPosition);

		if ($Mode == cConstants::PB_String_NoCase) {
            $p2 = preg_replace('/'.$StringToFind.'/i', $StringToReplace, $p2, $NbOccurrences);
		} else {
            $p2 = preg_replace('/'.$StringToFind.'/', $StringToReplace, $p2, $NbOccurrences);
		}

		return $p1 . $p2;
        
    }

    // Result$ = ReverseString(String$)
    static function ReverseString($String) {
        return strrev($String);
    }

    // Result$ = Right(String$, Length)
    static function Right($String, $Length) {
        return substr($String, strlen($String) - $Length);
    }

    // Result$ = Space(Length)
    static function Space($Length) {
        return str_repeat(" ", $Length);
    }

    // Result$ = Str(Value.q)
    static function Str($Value) {
        return strval($Value);
    }

    // Result$ = StrD(Value.d [, NbDecimal])    
    static function StrD($Value, $NbDecimal = -1) {
        if ($NbDecimal==-1) return strval($Value);
        return number_format($Value, $NbDecimal);
    }

    // Result$ = StrF(Value.f [, NbDecimal])
    static function StrF($Value, $NbDecimal = -1) {
        if ($NbDecimal==-1) return strval($Value);
        return number_format($Value, $NbDecimal);
    }
    
    // Result$ = StrU(Value.q, Type)
    static function StrU($Value, $Type) {
        // ToDo
    }

    // Result = StringByteLength(String$ [, Format])
    static function StringByteLength($String, $Format) {
        return strlen($String);
    }

    // Result$ = StringField(String$, Index, Delimiter$)
    static function StringField($String, $Index, $Delimiter) {
        $Index -= 1;
        $myArray = explode($Delimiter, $String);
        if (sizeof($myArray) >= $Index) {
            return $myArray[$Index];
        } else {
            return "";
        }
    }

    // Result$ = Trim(String$ [, Character$])
    static function Trim($String, $Character = "") {
        return trim($String, $Character);
    }

    // Result$ = UCase(String$)
    static function UCase($String) {
        return strtoupper($String);
    }

    // Result.q = Val(String$)
    static function Val($String) {
        return intval($String);
    }
    
    // Result.d = ValD(String$)
    static function ValD($String) {
        return doubleval($String);
    }
    
    // Result.f = ValF(String$)
    static function ValF($String) {
        return floatval($String);
    }

}

?>